;**********************************
; Generate VFE diagram
; The script is adapted from taylor_4.ncl
;**********************************
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_code.ncl"   
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_csm.ncl"   
load "./VFE_diagram.ncl"
;load "./VFE_metrics_table.ncl"
;**********************************
begin
;**********************************
; Assume the following have already been computed:
;   _ratio are the ratio:  Case_RMSL/Reference_RMSL
;   _cc    are the vector similarity coef of Case to Reference
;   _std   are the standard deviation of RMS values of various variables
;**********************************

; Cases 
  case      = (/ "CMIP5 Models", "Observations"/)
  nCase     = dimsizes(case )                 ; # of Cases [Cases]

; Models compared
  var       = (/"Model 1", "Model 2", "Model 3", "Model 4", "Model 5", "Model 6", "Model 7", "Model 8", "Model 9"/) 
  nVar      = dimsizes(var)                   ; # of Variables

; MIEI values for various models
  MIEI      = (/"0.34", "0.44", "0.48", "0.44", "0.51", "0.35", "0.37", "0.37", "0.46"/) 

; "Statistics derived from models and reference data"                        
  CA_ratio   = (/1.019, 1.057, 0.989, 0.942, 1.152, 0.958, 1.088, 1.027, 0.884/)
  CA_cc      = (/0.944, 0.900, 0.905, 0.921, 0.899, 0.938, 0.929, 0.929, 0.906/)
  CA_std     = (/0.054, 0.044, 0.200, 0.169, 0.191, 0.080, 0.030, 0.057, 0.130/)

; "Statistics derived from individual observational estimates and reference data"                        
  CB_ratio   = (/1.008, 1.001, 1.024, 3.0 ,3.0, 3.0, 3.0, 3.0, 3.0/)   ; the points (RMSL=3,Rv=1) do not occur in the diagram
  CB_cc      = (/0.987, 0.991, 0.986, 1.0 ,1.0, 1.0, 1.0, 1.0, 1.0/)
  CB_std     = (/0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0/)

; arrays to be passed to VFE_diagram. It will calculate the x xnd y coordinates.
  ratio      = new ((/nCase, nVar/),typeof(CA_cc) )  
  cc         = new ((/nCase, nVar/),typeof(CA_cc) ) 
  std        = new ((/nCase, nVar/),typeof(CA_std) )

  ratio(0,:) = CA_ratio 
  ratio(1,:) = CB_ratio

  cc(0,:)    = CA_cc 
  cc(1,:)    = CB_cc

  std(0,:)    = CA_std
  std(1,:)    = CB_std

;**********************************
; create plot
;**********************************
  MIEIs = "MIEI = " + MIEI              ; add extra info [*not* required]

  ty_opt   = True                           ; taylor diagram with options
        
  ty_opt@Markers       = (/20, 6/)         ; make all solid fill
;  ty_opt@Colors        = (/"red", "blue"/)          
  ty_opt@varLabels     = MIEIs
  ty_opt@caseLabels    = case
  ty_opt@Colors  = (/"blue","orangered","darkgreen","black","cyan4",\
                     "purple3","steelblue","red1","brown"/)

  ty_opt@varLabelsYloc = 0.66               ; Move location of MIEI values. negetative => not showing model name 
  ty_opt@caseLabelsFontHeightF = 0.12       ; make slight larger        [default=0.12 ]
  ty_opt@varLabelsFontHeightF  = 0.013      ; font size of model name   [default=0.013]
  ty_opt@gsMarkerSizeF         = 0.016      ; marker size               [default=0.0085]
  ty_opt@gsMarkerThicknessF    = 0.5
  ty_opt@NumberSizeF           = 0.013        ; font size of model number
  ty_opt@txFontHeightF         = 0.018      ; text size of axises
  ty_opt@markerTxYOffset       = 0.0        ; [default=0.0175]
 
  ty_opt@tiMainString  = ""           ; title
  ty_opt@stnRad        = (/ 0.5,1.5 /)      ; additional standard radii
  ty_opt@ccRays        = (/ 0.7,0.95 /)     ; correllation rays
  ty_opt@centerDiffRMS = True               ; RMS 'circles'
  ty_opt@centerDiffRMS_color = "LightGray"  ; default is "black"
;  ty_opt@ccRays_color    = "LightGray"       ; default is "black"

  wks   = gsn_open_wks("ps","./graph/VFE.T-P.uncentered.3obs.9models.withbars")
  plot  = VFE_diagram(wks,ratio,cc,std,ty_opt)

;**************************************************
; fill an array for a "taylor metrics table"
;**************************************************

  season    = (/ "ANN" /)
  nSeason   = dimsizes(season)

  table     = new ( (/nCase,nSeason,nVar/), typeof(ratio) )
  table(0,0,:) = CA_ratio
  table(1,0,:) = CB_ratio

  tt_opt         = True
  tt_opt@pltType = "ps"                  ; "eps" [default], "pdf", "ps"
                                         ; "png", "gif" [if you have ImageMajik 'convert']
  tt_opt@color0  = "palegreen2"
  tt_opt@color1  = "tomato2"

end 
