;**********************************
; VFE metrics
; Generate a table showing multiple statistics of multiple variables
; Xu Z., Y. Han, C. Fu, Geosci. Model Dev. Discuss. https://doi.org/10.5194/gmd-2017-95, 2017
; Xu Z., Z. Hou, Y. Han, W. Guo, Geosci. Model Dev., 9, 4365-4380, 2016
;**********************************
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_code.ncl"   
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_csm.ncl"   
;load "./VFE_diagram.ncl"
load "./VFE_metrics_table.ncl"
;**********************************
begin
;**********************************
; Assume the following have already been computed:
;  RMSvar:    RMS values of various scalar fields
;  CORRvar:   uncentered/centered correlation coefficients of modeled fields with observational estimates
;  RMSDvar:   RMSDs between modeled fields and observational estimates
;  MIEI:      multi-variable integrated evaluation index
;**********************************


; Statistics

  statistics  = (/ "RMS", "CORR","RMSD"/)
  nStatistics = dimsizes(statistics)                 ; # of Statistics

; Models
  model      = (/"Model 1", "Model 2", "Model 3", "Model 4", "Model 5", "Model 6", "Model 7", "Model 8", "Model 9", "Obs1", "Obs2", "Obs3"/) 
  nmodel      = dimsizes(model)                   ; # of Models


;**************************************************
; fill arrays for a "VFE metrics table"
;**************************************************

  vars    = (/ "Tm", "Pm", "Ta", "Pa" /)            ; variable name
  nVars   = dimsizes(vars)                          ; # of variables
  RMSvar      = new ((/nVars, nmodel/),float )  
  RMSvar(0,:) = (/1.01,  1.04,  1.05,  0.97,  0.99,  1.05,  1.06,  1.02,  0.92, 1.00,  0.99,  1.01/)
  RMSvar(1,:) = (/0.99,  1.01,  0.80,  0.84,  0.94,  0.97,  1.09,  0.97,  0.91, 0.99,  1.01,  1.02/)
  RMSvar(2,:) = (/1.11,  1.13,  1.26,  1.17,  1.43,  0.97,  1.14,  1.12,  1.01, 0.97,  0.99,  1.10/)
  RMSvar(3,:) = (/0.97,  1.05,  0.77,  0.72,  1.19,  0.83,  1.07,  1.00,  0.66, 1.07,  1.00,  0.97/)

  CORRvar      = new ((/nVars, nmodel/),float )  
  CORRvar(0,:) = (/1.00,  1.00,  1.00,  1.00,  1.00,  1.00,  1.00,  1.00,  0.99, 1.00,  1.00,  1.00/)
  CORRvar(1,:) = (/0.92,  0.83,  0.88,  0.91,  0.86,  0.90,  0.90,  0.89,  0.86, 0.99,  0.99,  1.00/)
  CORRvar(2,:) = (/0.95,  0.93,  0.93,  0.95,  0.95,  0.96,  0.93,  0.96,  0.93, 0.98,  0.99,  0.97/)
  CORRvar(3,:) = (/0.92,  0.84,  0.86,  0.87,  0.84,  0.90,  0.89,  0.87,  0.87, 0.98,  0.98,  0.99/)

  RMSDvar      = new ((/nVars, nmodel/),float )  
  RMSDvar(0,:) = (/0.09,  0.10,  0.11,  0.09,  0.10,  0.09,  0.11,  0.08,  0.14, 0.03,  0.04,  0.04/)
  RMSDvar(1,:) = (/0.41,  0.58,  0.48,  0.43,  0.51,  0.43,  0.47,  0.46,  0.51, 0.13,  0.11,  0.09/)
  RMSDvar(2,:) = (/0.35,  0.42,  0.48,  0.38,  0.58,  0.29,  0.41,  0.33,  0.37, 0.18,  0.16,  0.27/)
  RMSDvar(3,:) = (/0.39,  0.58,  0.52,  0.52,  0.64,  0.44,  0.48,  0.51,  0.54, 0.21,  0.18,  0.16/)

  table     = new ( (/nStatistics,nVars,nmodel/), float )
;*** NOTE: the RMS values, CORR, and RMSD should be assigned to the 0, 1, 2 dimension, respectively.
  table(0,:,:) = RMSvar
  table(1,:,:) = CORRvar
  table(2,:,:) = RMSDvar
;**************************************************
; fill arrays of RMS_std and MIEI "
;**************************************************
    RMSL    = (/1.02, 1.06, 0.99, 0.94, 1.15, 0.96, 1.09, 1.03, 0.88, 1.01, 1.00, 1.02/)
    VSC     = (/0.94, 0.90, 0.91, 0.92, 0.90, 0.94, 0.93, 0.93, 0.91, 0.99, 0.99, 0.99/)
    RMSVD   = (/0.34, 0.46, 0.43, 0.39, 0.50, 0.34, 0.40, 0.38, 0.42, 0.16, 0.13, 0.17/)
    RMSstd  = (/0.05, 0.04, 0.20, 0.17, 0.19, 0.08, 0.03, 0.06, 0.13, 0.04, 0.01, 0.05/)
    MIEI    = (/0.34, 0.44, 0.48, 0.44, 0.51, 0.35, 0.37, 0.37, 0.46, 0.23, 0.22, 0.24/)

    VFEvalues      = new ((/5, nmodel/),float )
    VFEvalues(0,:) = RMSL
    VFEvalues(1,:) = VSC 
    VFEvalues(2,:) = RMSVD
    VFEvalues(3,:) = RMSstd
    VFEvalues(4,:) = MIEI
 
    tt_opt         = True
    tt_opt@pltType = "ps"                           ; "eps" [default], "pdf", "ps"
;   tt_opt@wkOrientation  = "landscape"
    tt_opt@wkOrientation  = "portrait"

    zoom = 0.73           ; scaling the width of the table colomn
    VFE_metrics_table("./graph/VFE.metrics.T-P.uncentered",model,zoom,vars,table,VFEvalues,tt_opt)
end 

